package com.sky.tvheadend_manager;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Collections;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap.KeySetView;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.sky.tvheadend_manager.Muxes.ServiceInfo;
import com.sky.tvheadend_manager.TvheadendInfos.ServiceInner;

public class ChannelsManager {
	public static final String UUID_MODE = "uuid";
	public static final String PROVIDER_MODE = "provid";
	public static final String UNENCRYPTED = "clair";
	
	public static final String DEL_MODE_TAG = "tag";
	public static final String DEL_MODE_CHANNEL = "channel";
	public static final String DEL_MODE_SERVICE = "serv";
	
	
	public static final String CHECK_CHANNEL = "channel";
	public static final String CHECK_FIX = "fix";
	
	
	public static final String TAG_FOLDER = "/channel/tag/";
	public static final String CH_CONFIG_FOLDER = "/channel/config/";
	
	
	public static Tag createTag(String name){
		TvheadendInfos infos = TvheadendInfos.getInstance();
		Tag tag =infos.tagsList.values().parallelStream()
			.filter(t->t.name.equals(name))
			.findAny()
			.orElse(null);
		if(tag!=null){
			System.out.println("Tag already exists");
			return tag;
		}
		
		Tag nTag = new Tag();
		nTag.internal = false;
		nTag.comment ="";
		nTag.enabled = true;
		nTag.name = name;
		nTag.private_bool = false;
		nTag.index = 0;
		nTag.icon="";
		nTag.title_icon=false;
		nTag.tagUUID = UUID.randomUUID().toString().replace("-", "");
		Gson gson = new GsonBuilder().setPrettyPrinting().create();
		String json = gson.toJson(nTag);
	
		if(Utils.writeJsonToFile(json, infos.parent_folder+TAG_FOLDER+nTag.tagUUID)){
			if(!infos.tagsList.containsKey(nTag.tagUUID)){
			infos.tagsList.put(nTag.tagUUID,nTag);
			infos.fireTagChangedListener();
			}
			return nTag;
		}
		
		return null;
	}
	
	public static void addChannel(String name,String tagName,String mode){
		int mode_int = 0;System.out.println("add channel "+name+" "+tagName+" "+mode);
		if(mode!=null){
			switch(mode){
			case UUID_MODE:	mode_int = 1;break;
			case PROVIDER_MODE: addChannelWithProverdId(name,tagName);return;
			case UNENCRYPTED: addUnencryptedChannels(tagName);return;
			default: mode_int = 0;break;
			}
		}
		String serv_uuid;
		TvheadendInfos infos = TvheadendInfos.getInstance();
		ChannelConfig ch_conf;
		switch(mode_int){
		case 1: serv_uuid = name;ch_conf = infos.findChannel(serv_uuid);break;
		default:
			serv_uuid = infos.findServiceUUIDByChannelName(name);
			if(serv_uuid==null){
				System.out.println("no service matching");
				return;
			}
			ch_conf = infos.findChannel(serv_uuid);
			break;
		}
		if(ch_conf==null){
			ch_conf = new ChannelConfig();
			ch_conf.bouquet ="";
			ch_conf.configUUID = UUID.randomUUID().toString().replace("-", "");
			ch_conf.dvr_pre_time = 0;
			ch_conf.dvr_pst_time = 0;
			ch_conf.enabled = true;
			ch_conf.epg_running = -1;
			ch_conf.number = 0;
			ch_conf.services = new ArrayList<>();
			ch_conf.tags = new ArrayList<>();
		}

		Tag ntag = infos.getTagByName(tagName);
		
		boolean newServ = false;
				
		if((ntag!=null)){
			if( getTag(ch_conf, ntag.tagUUID)==null){

				ch_conf.tags.add(ntag.tagUUID);

			}
			
		}else{
			ntag= createTag(tagName);
			if(ntag!=null){
				ch_conf.tags.add(ntag.tagUUID);
			}else{
				return;
			}
			ch_conf.tags.add(ntag.tagUUID);
		}
		if(!serviceExistChannel(ch_conf, serv_uuid)){
			ch_conf.services.add(serv_uuid);
			newServ = true;
		}
		
		addChannel(ch_conf);
		
		
	}
	
	public static void delete(String mode, String param1,String param2){
		switch(mode){
		case DEL_MODE_TAG:
			deleteTag(param1);
			break;
		case DEL_MODE_SERVICE:
			deleteServiceFromTag(param1,param2);break;
			default:break;
		}
	}
	public static String checkConfig(String config,boolean fix){
		if(config==null)
			config = CHECK_CHANNEL;
		switch(config){

		case CHECK_CHANNEL:
			return checkChannelConfig(fix);
			
		default:return checkChannelConfig(fix);

		}
	}
	
	private static boolean serviceExistChannel(ChannelConfig ch_conf,String serv_uuid){
		String serv=ch_conf.services.parallelStream()
			.filter(s->s.equals(serv_uuid))
			.findAny()
			.orElse(null);
		if(serv!=null)
			return true;
		
		return false;
	}
	
	private static Tag getTag(ChannelConfig ch_conf,String tagUUID){
		if(tagUUID!=null){
			String t_ret = ch_conf.tags.parallelStream()
				.filter(tUUID->tUUID.equals(tagUUID))
				.findAny()
				.orElse(null);
			if(t_ret!=null){
				TvheadendInfos infos = TvheadendInfos.getInstance();
				Tag tag =infos.tagsList.get(tagUUID);
				if(tag!=null){
					return tag;
				}
			}
			
		}
		return null;
	}
	
	private static void addChannelWithProverdId(String provId, String tag){
		if(provId==null){
			System.out.println("provider id null");
			return;
		}
		int id = Integer.decode(provId);
		TvheadendInfos infos = TvheadendInfos.getInstance();
		ArrayList<ServiceInfo> servList = infos.findChannelsWithProverId(id);
		for(ServiceInfo serv_info:servList){
			addChannel(serv_info.serviceUUID, tag, UUID_MODE);
		}
	}
	
	private static void addUnencryptedChannels(String tag){
		TvheadendInfos infos = TvheadendInfos.getInstance();
		for(ServiceInfo serv_info:infos.getListUnencrypted()){
			addChannel(serv_info.serviceUUID, tag, UUID_MODE);
		}
	}
	
	private static String checkChannelConfig(boolean fix){
		TvheadendInfos infos = TvheadendInfos.getInstance();
		StringBuilder stB = new StringBuilder();
		for(ChannelConfig ch_conf:infos.chConfigList.values()){
			stB.append(checkUniqueTagAndService(ch_conf,fix));
			stB.append(checkUniqueConfForService(ch_conf,fix));
			stB.append(checkExistingTag(ch_conf,fix));
			if(fix && stB.length()>0){
				boolean success = Utils.writeJsonToFile(new Gson().toJson(ch_conf),infos.parent_folder+CH_CONFIG_FOLDER+ch_conf.configUUID);
				if(success){
					System.out.println("Write conf "+ch_conf.configUUID +" file ");
				}
			}
			
		}
		if(stB.length()==0){
			return "No error";
		}
		return stB.toString();
	}
	
	
	private static boolean isTagExist(String tagUUID,String tagName){
		TvheadendInfos infos = TvheadendInfos.getInstance();
		Tag tag = infos.getTag(tagUUID);
		if(tag!=null && tag.name.equals(tagName))
			return true;
			else
				
			return false;
	}
	
	
	private static boolean deleteTag(String tagName){
		TvheadendInfos infos = TvheadendInfos.getInstance();
		Tag tag = infos.getTagByName(tagName);
		boolean faileddelete = false;
		if(tag!=null){
			Tag tagtmp ;
			for(ChannelConfig ch_conf:infos.chConfigList.values()){
				if((tagtmp=getTag(ch_conf, tag.tagUUID))!=null){
					if(ch_conf.tags.remove(tagtmp.tagUUID)){
						checkUniqueTagAndService(ch_conf,true);
						if(Utils.writeJsonToFile(new Gson().toJson(ch_conf), infos.parent_folder+CH_CONFIG_FOLDER+ch_conf.configUUID)){
							System.out.println(tagtmp.name +" tag has been removed from "+ch_conf.configUUID);
							continue;
						}
					}
					System.out.println(" Failed to remove tag "+tagtmp.name +"  from "+ch_conf.configUUID);
					faileddelete = true;
				}
			}
			if(!faileddelete){
				
				if(infos.tagsList.remove(tag.tagUUID)!=null){
					File f = new File(infos.parent_folder+TAG_FOLDER+tag.tagUUID);
					if(f.delete()){
						System.out.println(tag.name +" tag has been delete ");
						//infos.tagsList.remove(tag);
						infos.fireTagChangedListener();
						return true;
					}
				}
				System.out.println("failed to delete tag "+tag.name);
			}
		}
		return false;
	}
	
	private static String checkUniqueTagAndService(ChannelConfig ch_config,boolean fix){
		ArrayList<String> seen = new ArrayList<>();
		StringBuilder stB = new StringBuilder();
		for(int i=0;i<ch_config.services.size();){
			String servUUId = ch_config.services.get(i);
			if(seen.contains(servUUId)){
				stB.append("multiple service " +servUUId+ " exist on "+ch_config.configUUID+"\n");
				if(fix){
					if(ch_config.services.remove(i)!=null){
						stB.append("Fix \n");
					}else{
						stB.append("Failed Fix\n");
					}
					continue;
				}

			}else{
				seen.add(servUUId);

			}
			i++;
		}
		seen.clear();
		for(int i=0;i<ch_config.tags.size();){
			String tagUUID = ch_config.tags.get(i);
			if(seen.contains(tagUUID)){
				stB.append("multiple tag " +tagUUID+ " exist on "+ch_config.configUUID+"\n");
				if(fix){
					if(ch_config.tags.remove(i)!=null){
						stB.append("Fix \n");
					}else{
						stB.append("Failed Fix\n");

					}
					continue;
				}

			}else{
				seen.add(tagUUID);

			}
			i++;
		}
		return stB.toString();
	}
	
	private static String checkUniqueConfForService(ChannelConfig ch_config,boolean fix){
		TvheadendInfos infos = TvheadendInfos.getInstance();
		StringBuilder stB = new StringBuilder();
		for(String servUUID:ch_config.services){
			ArrayList<String> keys=Collections.list(infos.chConfigList.keys());
			for(int i=0;i<keys.size();){
				ChannelConfig ch_conf = infos.chConfigList.get(keys.get(i));
				if(!ch_conf.equals(ch_config)){
					if(ch_conf.services.contains(servUUID)){
						stB.append(servUUID +" exist on "+ch_conf.configUUID+" and "+ch_config.configUUID+"\n");
						if(fix){
							ch_config.append(ch_conf);
							if(infos.chConfigList.remove(ch_conf.configUUID)!=null){
								File f = new File(infos.parent_folder+CH_CONFIG_FOLDER+ch_conf.configUUID);
								if(f.delete()){
									System.out.println(ch_conf.configUUID+" config has been delete ");
									stB.append("Fix \n");
									infos.fireChannelConfigChangedListener();
									keys=Collections.list(infos.chConfigList.keys());
									i=0;
									continue;
									
								}
							}
						return stB.append("Failed Fix \n").toString();
						}
					}
				}
				i++;
			}
		}
		return stB.toString();
	}
	
	private static String checkExistingTag(ChannelConfig ch_config, boolean fix){
		TvheadendInfos infos = TvheadendInfos.getInstance();
		StringBuilder stB = new StringBuilder();
		int i;
		String tag_str;
		Tag tag;
		for(i=0;i<ch_config.tags.size();){
			tag_str = ch_config.tags.get(i);
			if(infos.getTag(tag_str)==null){
				System.out.println("Tag with UUID "+tag_str+" is missing ["+ch_config.configUUID+"]");
				stB.append("Tag with UUID "+tag_str+" is missing ["+ch_config.configUUID+"]");
				if(fix){
					if(ch_config.tags.remove(i)!=null){
						
							System.out.println("Fix ");
							continue;
						
					}
					System.out.println("failed to fix ");
					continue;
				}
			}
			i++;
		}
		return stB.toString();
	}
	private static void deleteServiceFromTag(String serviceUUID,String tagName){
		System.out.println(" deleteServiceFromTag " +serviceUUID +" from tag "+tagName);
		TvheadendInfos infos = TvheadendInfos.getInstance();
		ChannelConfig ch_conf = infos.findChannel(serviceUUID);
		if(ch_conf!=null){
			Tag tag = infos.getTagByName(tagName);
		
			if(ch_conf.tags.remove(tag.tagUUID)){
				
				if(Utils.writeJsonToFile(new Gson().toJson(ch_conf), infos.parent_folder+CH_CONFIG_FOLDER+ch_conf.configUUID)){
					infos.serviceMap.get(serviceUUID).chConf = null;
					infos.fireChannelConfigChangedListener();
					System.out.println(serviceUUID +"  has been removed from tag "+tag.name);
					
				}
			}
		}
	}
	public static void addChannel(ChannelConfig chConfig){
		TvheadendInfos infos = TvheadendInfos.getInstance();
		Gson gson = new GsonBuilder().setPrettyPrinting().create();
		String json = gson.toJson(chConfig);

		checkUniqueTagAndService(chConfig,true);
		if(Utils.writeJsonToFile(json, infos.parent_folder+CH_CONFIG_FOLDER+chConfig.configUUID)){
			infos.chConfigList.put(chConfig.configUUID,chConfig);
			if(infos.chConfigList.size()>0){
				String servUUID = chConfig.services.get(0);
				if(servUUID!=null){
					ServiceInner servIN =infos.serviceMap.get(servUUID);
					if(servIN!=null){
						servIN.chConf = chConfig;
					}
				}
			}
			
			infos.fireChannelConfigChangedListener();
		}
	}
}
