package com.sky.tvheadend_manager;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.internal.LinkedTreeMap;
import com.sky.tvheadend_manager.Muxes.ServiceInfo;
import com.sky.tvheadend_manager.Muxes.Stream;

import javafx.concurrent.Worker;

public class TvheadendInfos {
	
	protected ConcurrentHashMap<String,Tag> tagsList = new ConcurrentHashMap<>();
	protected ConcurrentHashMap<String,ChannelConfig> chConfigList = new ConcurrentHashMap<>();
	protected HashMap<String,ChannelConfig> chConfigListOnline = new HashMap<>();
	protected ConcurrentHashMap<String,ServiceInner> serviceMap = new ConcurrentHashMap<>();
	protected HashMap<String,List<Muxes>> mapMuxes =new HashMap<>();
	protected List<String> networksList = Collections.synchronizedList(new ArrayList<String>());
	final static String DEFAULT_TVHEADEND_FOLDER = "/home/hts/.hts/tvheadend";
	protected File parent_folder = new File(DEFAULT_TVHEADEND_FOLDER);
	 String tvheadenServer = "http://127.0.0.1";
	final static String NAME_MODE = "name"; 
	final static String UNENCRYPTED = "clair"; 
	final static String PROVIDERID = "provid"; 
	private static TvheadendInfos instance= null;
	
	 static {  
		 final String libname = "libtvheadendparser.so";
		try{
			 System.load("/home/angelofskies/workspace/tvheadendManager/files/"+libname);
		}catch (UnsatisfiedLinkError e){
			e.printStackTrace();
	        try {    
	            NativeUtils.loadLibraryFromJar("/"+libname);   
	        } catch (IOException ex) {
	            // This is probably not the best way to handle exception :-) 
	        	
	            ex.printStackTrace();
	        } 
		}
	      
	 }
	private TvheadendInfos(){
		
	}
	
	public class ServiceInner{
		ServiceInfo serv;
		ChannelConfig chConf;
		Muxes	mux;
	}
	boolean tagsListPopulated = false;
	boolean chConfigListPopulated = false;
	boolean mapMuxesListPopulated = false;
	boolean networksListPopulated = false;
	
	protected ArrayList<TvheadenInfosChangeListener> infosChangedListener = new ArrayList<>();
	
	
	protected void fireTagChangedListener(){
		for(TvheadenInfosChangeListener listener:infosChangedListener){
			listener.onTagChanged();
		}
	}
	protected void fireNetworksChangedListener(){
		for(TvheadenInfosChangeListener listener:infosChangedListener){
			listener.onNetworksChanged();
		}
	}
	
	protected void fireMuxesChangedListener(){
		for(TvheadenInfosChangeListener listener:infosChangedListener){
			listener.onMuxesChanged();
		}
	}
	
	protected void fireChannelConfigChangedListener(){
		for(TvheadenInfosChangeListener listener:infosChangedListener){
			listener.onChannelConfigChanged();
		}
	}
	public void addTvheadendInfoChangedListener(TvheadenInfosChangeListener listener){
		if(listener!=null && !infosChangedListener.contains(listener)){
			infosChangedListener.add(listener);
			if(tagsListPopulated)
			listener.onTagChanged();
			if(networksListPopulated)
				listener.onNetworksChanged();
			if(mapMuxesListPopulated)
				listener.onMuxesChanged();
		}
	}
	public static TvheadendInfos getInstance(){
		if(instance==null){
			instance = new TvheadendInfos();
			instance.populateInfo();
		}
		return instance;
	}
	
	
	public void populateInfo(){
		tagsListPopulated = false;
		chConfigListPopulated = false;
		mapMuxesListPopulated = false;
		networksListPopulated = false;
		
		chConfigList.clear();
		tagsList.clear();
		networksList.clear();
		mapMuxes.clear();
		serviceMap.clear();
		
				
				chConfigList = populateChannelConfigList();
				chConfigListPopulated = true;
				fireChannelConfigChangedListener();
		

				tagsList = populateTagList();
				tagsListPopulated = true;
				fireTagChangedListener();
			
	
				File networks = Utils.getFile(parent_folder, new String[]{"input","dvb","networks"});
				if(networks!=null && networks.isDirectory()){
					for(String netw:networks.list()){
						networksList.add(netw);
						mapMuxes.put(netw, populateMuxesList( netw));
					}
					
				}
				networksListPopulated = true;
				mapMuxesListPopulated = true;
				fireNetworksChangedListener();
				fireMuxesChangedListener();
				
				for(ChannelConfig ch_conf:chConfigList.values()){
					for(String servUUID:ch_conf.services){
						ServiceInner servIN=serviceMap.get(servUUID);
						if(servIN!=null){
							servIN.chConf = ch_conf;
						}
					}
				}
				
				getServerChannelList();
			

	}
	
	public HashMap<String, ChannelConfig> getServerChannels(){
		return chConfigListOnline;
	}
	public ConcurrentHashMap<String,Tag> getTagList(){
		return tagsList;
	}
	private  ConcurrentHashMap<String,Tag> populateTagList(){
		
		tagsList.clear();
		File tag_folder = Utils.getFile(parent_folder,new String[]{"channel","tag"});


		if(tag_folder!=null && tag_folder.isDirectory()){
			ExecutorService executor = Executors.newFixedThreadPool(5);
			String[] tags_files = tag_folder.list();
			for(String tagUUID:tags_files){
				executor.execute(new Runnable(){
					public void run(){
						File file_tag = new File(tag_folder.getAbsolutePath()+"/"+tagUUID);
						Tag tag = Utils.parseJson(file_tag,Tag.class,false);
						if(tag!=null){
							tag.tagUUID = tagUUID;
							tagsList.put(tag.tagUUID,tag);
						}
					}
				});
				
			}
			executor.shutdown();
	        while (!executor.isTerminated()) {
	        }
		}
		
		return tagsList;
	}
	
	public ConcurrentHashMap<String,ChannelConfig> getChannelConfigList(){
		return chConfigList;
	}
	private  ConcurrentHashMap<String,ChannelConfig> populateChannelConfigList(){
		
		chConfigList.clear();
		File ch_configFolder = Utils.getFile(parent_folder,new String[]{"channel","config"});
		if(ch_configFolder!=null && ch_configFolder.isDirectory()){
			ExecutorService executor = Executors.newFixedThreadPool(5);
			String[] config_files = ch_configFolder.list();
			for(String confUUID:config_files){
				executor.execute(new Runnable(){
					
					public void run(){
						File file_config = new File(ch_configFolder.getAbsolutePath()+"/"+confUUID);
						ChannelConfig ch_conf = Utils.parseJson(file_config,ChannelConfig.class,false);
						if(ch_conf!=null){
							ch_conf.configUUID = confUUID;
							chConfigList.put(ch_conf.configUUID,ch_conf);
						}
					}
				});
				
			}
			executor.shutdown();
	        while (!executor.isTerminated()) {
	        }
		}
		
		
		return chConfigList;
	}
	
	public List<String> getNetworksList(){
		return networksList;
	}
	public List<Muxes> getMuxesList(String network){
		return mapMuxes.get(network);
	}

	private  List<Muxes> populateMuxesList(String network){

		
		List<Muxes> muxes = Collections.synchronizedList(new ArrayList<Muxes>());
		File muxes_folder = Utils.getFile(parent_folder,new String[]{"input","dvb","networks",network,"muxes"});
		if(muxes_folder!=null && muxes_folder.isDirectory()){
			ExecutorService executor = Executors.newFixedThreadPool(5);
			String[] mux_files = muxes_folder.list();
			for(String mux_f:mux_files){
				executor.execute(new Runnable(){
					public void run(){
						File mux = new File(muxes_folder.getAbsolutePath()+"/"+mux_f);

						Object obj = Utils.parseJson(mux,Object.class,true);
						if(obj!=null){
							String json =new Gson().toJson(obj);
							Muxes ret = new Gson().fromJson(json, Muxes.class);
							if(ret!=null){
								ret.config.service = ret.newService();
								Object config_obj = ((LinkedTreeMap<Object, Object>)obj).get("config");
								Object serv_obj = ((LinkedTreeMap<Object, Object>)config_obj).get("services");
								if(((LinkedTreeMap<Object, Object>)serv_obj).size()>0){
									for(String key:((LinkedTreeMap<String, Object>)serv_obj).keySet()){

										Object serv_info = ((LinkedTreeMap<String, Object>)serv_obj).get(key);
										if(serv_info!=null){
											String json_srv = new Gson().toJson(serv_info);
											ServiceInfo srv=new Gson().fromJson(json_srv, Muxes.ServiceInfo.class);
											ServiceInner servIN = new ServiceInner();
											servIN.serv = srv;
											servIN.mux = ret;
											if(ret.config.service.serviceInfo.add(srv)){
												serviceMap.put(key,servIN );
												ret.config.service.serviceInfo.get(ret.config.service.serviceInfo.size()-1).serviceUUID = key;
											}
										}
									}

								}

								ret.muxUUID = mux_f;

								muxes.add(ret);
							}
						}

					}
				});

			}
			executor.shutdown();
			while (!executor.isTerminated()) {
				
			}
			
		}
		return muxes;
	}
	
	
	
	
	public void displayTagList(){
		for(Tag t:tagsList.values()){
			System.out.println(t.toString());
		}
	}

	public void displayChConfigList(String mode,String param){
		int mode_int = 0;
		if(mode!=null){
			switch(mode){
			case NAME_MODE: mode_int = 1;break;
			case UNENCRYPTED: displayUnencrypted();return;
			case PROVIDERID: displayProviderId(param);return;
			default: mode_int = 0;break;
			}
		}	
		
		StringBuilder strB = new StringBuilder();
		for(ChannelConfig ch_config:chConfigList.values()){

			strB.append("services : ");
			for(String serv:ch_config.services){
				ServiceInfo serv_info = findServiceInfos(serv);
				if (serv_info!=null){
					switch(mode_int){
					case 1:strB.append(serv_info.svcname+" ["+serv_info.serviceUUID+"]");break;
					default:strB.append(new Gson().toJson(serv_info));break;
					}
					
				}else{
					strB.append(serv);
				}
				strB.append(",");
			}
			strB.append("\n");
			strB.append("tags : ");
			for(String tag_str:ch_config.tags){
				strB.append(findTagName(tag_str));
				strB.append(",");
			}
			strB.append("\n");

		}
		System.out.println(strB.toString());
	}
	
	public ArrayList<ChannelConfig> getChannelsTagList(String tagName){

		ArrayList<ChannelConfig> ret = new ArrayList<>();
		if(tagName!=null){
			/*ret.addAll(chConfigList.values().stream()
				.flatMap(ch->ch.tags.stream())
				.filter(tagUUID->tagName.equals(findTagName(tagUUID)))
				.collect(Collectors.toList()));*/
			for(ChannelConfig ch_config:chConfigList.values()){
				for(String tagUUID:ch_config.tags){

					if(tagName.equals(findTagName(tagUUID))){
						ret.add(ch_config);

						break;
					}
				}

			}
		}
		return ret;
	}
	public void displayChTagList(String tagName){
		if(tagName!=null){
			StringBuilder strB = new StringBuilder();
			for(ChannelConfig ch_config:getChannelsTagList(tagName)){

				strB.append("services : ");
				for(String serv:ch_config.services){
					ServiceInfo serv_info = findServiceInfos(serv);
					if (serv_info!=null){
						strB.append(new Gson().toJson(serv_info));
					}else{
						strB.append(serv);
					}
					strB.append(",");
				}
				strB.append("\n");
				strB.append("tags : ");
				for(String tag_str2:ch_config.tags){
					strB.append(findTagName(tag_str2));
					strB.append(",");
				}
				strB.append("\n");

			}

			System.out.println(strB.toString());
		}
	}
	public void displayMuxesList(String network){
		if(network!=null){
			List<Muxes> muxes = mapMuxes.get(network);
			if(muxes!=null){
				muxes.stream()
					.forEach(m->System.out.println(new Gson().toJson(m)));
				
			}
		}
	}
	
	public void displayNetworksList(){
		for(String netw:networksList){
		System.out.println(netw);
		}
	}
	
	public Muxes getMux(String servUUID){
		if(servUUID!=null){
			for(String netw:networksList){
				List<Muxes> muxes = mapMuxes.get(netw);
				if(muxes!=null){
					for(Muxes mux:muxes){
						ServiceInfo f_serv= mux.config.service.serviceInfo.parallelStream()
							.filter(serv->serv.serviceUUID.equals(servUUID))
							//.map(x->{return mux;})
							.findAny()
							.orElse(null);
						if(f_serv!=null)
							return mux;
						
					}
				}
			}
		}
		return null;
	}
	
		
	public ArrayList<ServiceInfo> getListUnencrypted(){
		ArrayList<ServiceInfo> ret = new ArrayList<>();
		for(String netw:networksList){
			for(Muxes mux:mapMuxes.get(netw)){
				ret.addAll(mux.config.service.serviceInfo.parallelStream()
					.filter(serv->isUnencrypted(serv))
					.collect(Collectors.toList()));
				
			}
		}
		return ret;
	}
	
	public void displayUnencrypted(){

		getListUnencrypted().parallelStream()
			.forEach(serv->System.out.println(new Gson().toJson(serv)));
		

	}
	
	public ArrayList<ServiceInfo> findChannelsWithProverId(int proverId){
		ArrayList<ServiceInfo> ret = new ArrayList<>();
		for(String netw:networksList){
			for(Muxes mux:mapMuxes.get(netw)){
				for(ServiceInfo serv_info:mux.config.service.serviceInfo){
					for(Muxes.Stream stream:serv_info.stream){
						if(stream.caidlist!=null){
							for(Muxes.CAID caid:stream.caidlist){
								if(caid.providerid==proverId){
									ret.add(serv_info);
									break;
								}
							}
						}
					}
				}
			}
		}
		return ret;
	}
	private void displayProviderId(String provid){
		if(provid==null){
			System.out.println("provider id null");
			return;
		}
		int id = Integer.decode(provid);
		ArrayList<ServiceInfo> servList = findChannelsWithProverId(id);
		for(ServiceInfo serv_info:servList){
			System.out.println(new Gson().toJson(serv_info));
		}
		
	}
	public   String findServiceUUIDByChannelName(String name){
		if(name!=null){
			for(String netw:networksList){
				for(Muxes mux:mapMuxes.get(netw)){
					for(ServiceInfo serv_info:mux.config.service.serviceInfo){
						if(serv_info.svcname!=null && serv_info.svcname.equals(name)){
							return serv_info.serviceUUID;
						}
					}
				}
			}
		}
		return null;
	}
	public ServiceInfo findServiceInfos(String servUUID){
		if(servUUID!=null){
			ServiceInner servIN = serviceMap.get(servUUID);
			if(servIN!=null){
				return servIN.serv;
			}
		}
		return null;

	}
	public String findTagName(String tagUUID){
		if(tagUUID!=null){
				Tag tag =tagsList.get(tagUUID);
				if(tag!=null){
					return tag.name;
				}
			
		}
		return tagUUID;
	}
	
	public ChannelConfig findChannel(String serviceUUID){
		if(serviceUUID!=null){
			ServiceInner servIN=serviceMap.get(serviceUUID);
			if(servIN!=null){
				return servIN.chConf;
			}
			/*for(ChannelConfig ch_conf:chConfigList.values()){
				for(String serv:ch_conf.services){
					if(serv.equals(serviceUUID)){
						servIN.chConf = ch_conf;
						return ch_conf;
					}
				}
			}*/
		}
		return null;

	}
	
	private boolean isUnencrypted(ServiceInfo serv_info){
		///ServiceInfo serv_info = findServiceInfos(serviceUUID);
		boolean video_stream = false;
		if(serv_info!=null){
			for(Muxes.Stream stream:serv_info.stream){
				if(stream.width>0 && stream.height>0){
					video_stream = true;
				}
				if(stream.type.equals("CA"))
					return false;
			}
			
		}
		return video_stream;
	}
	
	public  Tag getTag(String tagUUID){
		if(tagUUID!=null){
			return tagsList.get(tagUUID);
		}
		return null;
	}
	
	
	public  Tag getTagByName(String tagName){
		if(tagName!=null){
			return tagsList.values().parallelStream()
				.filter(tag->tag.name.equals(tagName))
				.findAny()
				.orElse(null);
			/*for(Tag tag:tagsList.values()){
				if(tag.name.equals(tagName)){
					return tag;
				}
			}*/
		}
		return null;
	}
	
	
	public String getTvheadendFoler(){
		return parent_folder.getAbsolutePath();
	}
	public void setTvheadendFolder(String folder){
		if(folder!=null && !folder.equals(parent_folder.getAbsolutePath())){
		parent_folder = new File(folder);
		if(!parent_folder.exists()){
			System.out.println("set folder not exist");
			return;
		}
		if(!parent_folder.canWrite()){
			System.out.println("set folder not writable");
			return;
		}
		populateInfo();
		}
	}
	public String getTvheadendServer(){
		return tvheadenServer;
	}
	public void setTvheadendServer(String url){
		if(url!=null && !url.equals(tvheadenServer)){
			if(!url.startsWith("http://")){
				tvheadenServer="http://"+url;
			}else{
				tvheadenServer = url;
			}
		getServerChannelList();
		}
	}
	
	private void getServerChannelList(){
		new Thread(new Runnable() {

			@Override
			public void run() {
				chConfigListOnline.clear();
				String resp= WebServ.getResponseUrlConnection(tvheadenServer+":9981/api/channel/grid", WebServ.POST, "limit=999999999&all=1", null, null, null);
				JsonObject jObj= new JsonParser().parse(resp).getAsJsonObject();
				JsonArray jArr = jObj.get("entries").getAsJsonArray();
				ChannelConfig[] chArr = new Gson().fromJson(jArr, ChannelConfig[].class);

				for(ChannelConfig ch_conf:chArr){
					if(ch_conf.services.size()>0){
						ChannelConfig off_ch = findChannel(ch_conf.services.get(0));
						if(off_ch!=null){
							ch_conf.configUUID = off_ch.configUUID;
							chConfigListOnline.put(ch_conf.configUUID, ch_conf);
					
						}
					}
				}

			}
		}).start();
	}
}

