package com.sky.tvheadend_manager.ui;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.Properties;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.sky.tvheadend_manager.ChannelConfig;
import com.sky.tvheadend_manager.ChannelsManager;
import com.sky.tvheadend_manager.Muxes;
import com.sky.tvheadend_manager.Muxes.ServiceInfo;
import com.sky.tvheadend_manager.OSValidator;
import com.sky.tvheadend_manager.Tag;
import com.sky.tvheadend_manager.TvheadenInfosChangeListener;
import com.sky.tvheadend_manager.TvheadendFormatter;
import com.sky.tvheadend_manager.TvheadendInfos;

import javafx.application.Platform;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.collections.transformation.FilteredList;
import javafx.collections.transformation.SortedList;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.geometry.Side;
import javafx.scene.Node;
import javafx.scene.control.Accordion;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonType;
import javafx.scene.control.CheckBox;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.ComboBox;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.Dialog;
import javafx.scene.control.Label;
import javafx.scene.control.ListView;
import javafx.scene.control.MenuItem;
import javafx.scene.control.SelectionMode;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableColumn.SortType;
import javafx.scene.control.TableView;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.control.TitledPane;
import javafx.scene.control.ListCell;
import javafx.scene.input.ClipboardContent;
import javafx.scene.input.Dragboard;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.scene.input.TransferMode;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.util.Pair;

import org.controlsfx.control.CheckListView;
import org.controlsfx.control.ListSelectionView;;

public class ChannelListViewController {

	@FXML
	private TitledPane tagsList;
	@FXML
	private TitledPane networksList;
	
	@FXML 
	private TableView<ServiceInfo> tabChannels;
	@FXML 
	private TableColumn<ServiceInfo, String> chNumberColumn;
	 @FXML
	    private TableColumn<ServiceInfo, String> chNameColumn;
	 @FXML
	    private TableColumn<ServiceInfo, String> chSysColumn;
	 @FXML
	    private TableColumn<ServiceInfo, String> chFreqColumn;
	 @FXML
	    private TableColumn<ServiceInfo, String> chTagsColumn;
	 
	 @FXML
	 private TextField nameFilter;
	 @FXML
	 private TextField providerFiler;
	 @FXML
	 private CheckBox checkBoxUnencrypted;
	 
	private MainApp mainApp;
	private ObservableList<ServiceInfo> noFilterChannel;
	
	private ArrayList<String> dragedServices = new ArrayList<>();
	private final static String TAG_MODE = "tag";
	private final static String NETWORK_MODE = "net";
	private String tabMode = NETWORK_MODE;
	
	private String currentTag ;
	
	private TableColumn<ServiceInfo,String> sortcolumn;
	private SortType sortType;
	
		
	@FXML
	private void initialize(){
		TvheadendInfos infos = TvheadendInfos.getInstance();
		final ContextMenu contextMenu = new ContextMenu();
		MenuItem ctx_edit = new MenuItem("Edit");
		ctx_edit.setOnAction(event->editSelection());
		MenuItem ctx_add = new MenuItem("Add");
		ctx_add.setOnAction(event->showTagSelectionDialog());
		MenuItem ctx_remove = new MenuItem("Remove");
		ctx_remove.setOnAction(event->removeServiceFromTag());
		MenuItem ctx_play = new MenuItem("Play");
		ctx_play.setOnAction(event->playStream());
		contextMenu.getItems().addAll(ctx_edit,ctx_add,ctx_remove,ctx_play);
		tabChannels.setContextMenu(contextMenu);
		tabChannels.getSelectionModel().setSelectionMode(SelectionMode.MULTIPLE);
		tabChannels.setOnDragDetected(event -> {
			System.out.println("drag detected ");
			 Dragboard db = tabChannels.startDragAndDrop( TransferMode.COPY );
                ClipboardContent content = new ClipboardContent();
                for(ServiceInfo serv:tabChannels.getSelectionModel().getSelectedItems()){
                	content.putString(serv.getServiceUUID());
                }
          
                db.setContent( content );
                event.consume();
			
			});
		tabChannels.setOnMouseClicked(mouseEvent->{
		    
	        if(mouseEvent.getButton().equals(MouseButton.PRIMARY)){
	            if(mouseEvent.getClickCount() == 2){
	                
	                showServiceInfoDialog((((TableView<ServiceInfo>)mouseEvent.getSource()).getSelectionModel().getSelectedItem()));
	            }
	        }else if(mouseEvent.getButton().equals(MouseButton.SECONDARY)){
	        	contextMenu.show(tabChannels,mouseEvent.getScreenX(),mouseEvent.getScreenY());
		}
	    
	});
		
		
		infos.addTvheadendInfoChangedListener(new TvheadenInfosChangeListener() {
		
			@Override
			public void onTagChanged() {
				Platform.runLater(new Runnable(){
					public void run(){
						ListView<String> listTag = new ListView<>();
						ObservableList<String> data = FXCollections.observableArrayList();
						for(Tag tag:infos.getTagList().values()){
							 data.add(tag.toString());
							
						}
						listTag.setItems(data);
						listTag.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> showTagChannels(newValue));
						listTag.setOnMouseClicked(mouseEvent->{
						    
						        if(mouseEvent.getButton().equals(MouseButton.PRIMARY)){
						            if(mouseEvent.getClickCount() == 2){
						                 TvheadendInfos infos = TvheadendInfos.getInstance();
						               
						                mainApp.showTagEditDialog(infos.getTagByName(((ListView<String>)mouseEvent.getSource()).getSelectionModel().getSelectedItem()));
						            }
						        }
						    
						});
						listTag.setCellFactory(listView-> {
							final ListCell<String> cell = new ListCell<String>(){
							@Override
                            protected void updateItem(String tagName, boolean b) {
                                super.updateItem(tagName, b);
                                if(tagName != null) {
                                    setText(tagName);
                                }
							}
							};
					        cell.setOnMouseEntered(new EventHandler<MouseEvent>() {
					            @Override
					            public void handle(MouseEvent event) {
					                String tagName = cell.getItem();
					                if(tagName!=null && !tagName.isEmpty()){
					                for(String str:dragedServices){
					                	System.out.println(str);
					                	ChannelsManager.addChannel(str, tagName, ChannelsManager.UUID_MODE);
					                	
					                }
					                }
					                dragedServices.clear();
					            }
					        });
					        return cell;
						});
							listTag.setOnDragEntered( event ->

							{
								listTag.setStyle( "-fx-background-color: aqua;" );
							} );
							listTag.setOnDragExited( event ->
							{
								listTag.setStyle( "" );
							} );
							listTag.setOnDragOver( event  ->
							{

								Dragboard db = event.getDragboard();
								if ( db.hasString() )
								{
								event.acceptTransferModes( TransferMode.COPY_OR_MOVE );
								}
								event.consume();
							} );
							listTag.setOnDragDropped( event  ->
							{
								Dragboard db = event.getDragboard();
								boolean success = false;
								if ( db.hasString() )
								{
									dragedServices.add(db.getString());
									System.out.println( "Dropped: " + db.getString() );
									/*treeCell.getTreeItem().getChildren().add( new TreeItem<>( db.getString() ) );
				                    treeCell.getTreeItem().setExpanded( true );*/
									success = true;
								}
								
								event.setDropCompleted( success );
								event.consume();
							} );	

						
						updateTagTab(listTag);
					}
				});
			
			}

			
			@Override
			public void onNetworksChanged() {
				Platform.runLater(new Runnable(){
					public void run(){
						ListView<String> listNetworks = new ListView<>();
						ObservableList<String> dataNetw = FXCollections.observableArrayList();
						for(String netw:infos.getNetworksList()){
							dataNetw.add(netw);

						}
						listNetworks.setItems(dataNetw);
						networksList.setContent(listNetworks);
						listNetworks.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> showNetworksChannels(newValue));
					}
				});
			}

			@Override
			public void onMuxesChanged() {

				Platform.runLater(new Runnable(){
					public void run(){
						
						ObservableList<ServiceInfo> channelsData = FXCollections.observableArrayList();
						switch(tabMode){
						case NETWORK_MODE:
						
						
						for(String netw:infos.getNetworksList()){
							List<Muxes> muxes = infos.getMuxesList(netw);
							if(muxes!=null)
								for(Muxes mux:muxes){
									for(ServiceInfo serv:mux.getConfig().getService().getServiceInfo()){
										channelsData.add(serv);
									}
								}
						}
						break;
						
							default: break;
						}
						


						updateChannelTab(channelsData);
						noFilterChannel = channelsData;
					}
				});
			}

			@Override
			public void onChannelConfigChanged() {
				switch(tabMode){
				case TAG_MODE:
					showTagChannels(currentTag);
					break;
					default: break;
				
				}
				
			}
		});
		
		
		nameFilter.textProperty().addListener((observable, oldValue, newValue) -> {
		    filterByName(newValue);
		});	
		

		providerFiler.textProperty().addListener((observable, oldValue, newValue) -> {
		    filterByProvId(newValue);
		});	
		
		checkBoxUnencrypted.setOnAction(event->{
			if(checkBoxUnencrypted.selectedProperty().get()){
				saveSorting();
				ArrayList<ServiceInfo> servList=infos.getListUnencrypted();
				ObservableList<ServiceInfo> channelsData = FXCollections.observableArrayList();
			
					channelsData.addAll(servList);
				
				updateChannelTab(channelsData);
				filter = true;
				restoreSorting();
			}else{
				updateChannelTab(noFilterChannel);
				filter = false;
			}
		});
	}
	
	public void setMainApp(MainApp mainApp) {
        this.mainApp = mainApp;

        // Add observable list data to the table
        //personTable.setItems(mainApp.getPersonData());
    }
	 
	private void updateChannelTab(ObservableList<ServiceInfo> channelsData){
		tabChannels.setItems(channelsData);
		chNumberColumn.setCellValueFactory(cellData -> 
		getChannelNumber(cellData.getValue()));
		chNameColumn.setCellValueFactory(cellData -> 
		getChannelName(cellData.getValue()));
		chSysColumn.setCellValueFactory(cellData -> 
		getChannelSys(cellData.getValue()));
		chFreqColumn.setCellValueFactory(cellData -> 
		getChannelFreq(cellData.getValue()));
		chTagsColumn.setCellValueFactory(cellData -> 
		getChannelTags(cellData.getValue()));
		chTagsColumn.setCellFactory(TextAreaTableCell.forTableColumn()); // add StringConverter if neccessary
		//tabChannels.getColumns().add(chTagsColumn);
		
		
		
	}
	
	private void updateTagTab(ListView<String> listTag){
		VBox vBox = new VBox(2);
		HBox hBox = new HBox(2);
		Button btn_createTag = new Button("new");
		btn_createTag.addEventHandler(ActionEvent.ACTION, event -> mainApp.showTagEditDialog(null));
		Button btn_deleteTag = new Button("delete");
		btn_deleteTag.addEventHandler(ActionEvent.ACTION, event -> deleteTag(((ListView<String>)( (VBox) tagsList.getContent())
																					.getChildren().get(0))
																					.getSelectionModel().getSelectedItem()) );
		hBox.getChildren().addAll(btn_createTag,btn_deleteTag);
		vBox.getChildren().addAll(listTag,hBox);
		tagsList.setContent(vBox);
	}
	private void showTagChannels(String tagStr){
		//System.out.println("showTagChannels "+tagStr);
		currentTag = tagStr;
		tabMode = TAG_MODE;
		saveSorting();
		TvheadendInfos infos = TvheadendInfos.getInstance();
		updateChannelView(tagStr);
		restoreSorting();
	}
	
	private void showNetworksChannels(String netw){
		tabMode = NETWORK_MODE;
		if(netw!=null && !netw.isEmpty()){
			saveSorting();
			TvheadendInfos infos = TvheadendInfos.getInstance();
			ObservableList<ServiceInfo> channelsData = FXCollections.observableArrayList();
			for(Muxes mux:infos.getMuxesList(netw)){
				for(ServiceInfo serv:mux.getConfig().getService().getServiceInfo()){
					channelsData.add(serv);
				}
			}

			updateChannelTab(channelsData);
			noFilterChannel = channelsData;
			restoreSorting();
		}

	}
	
	private void showServiceInfoDialog(ServiceInfo serv){
		Alert alert = new Alert(AlertType.INFORMATION);
		alert.setTitle(serv.getSvcname());
		alert.setHeaderText("service uuid "+serv.getServiceUUID());
		Gson gson = new GsonBuilder().setPrettyPrinting().create();
		
		TextArea textArea = new TextArea(TvheadendFormatter.serviceInfoToJson(serv));
		textArea.setEditable(false);
		textArea.setWrapText(true);

		textArea.setMaxWidth(Double.MAX_VALUE);
		textArea.setMaxHeight(Double.MAX_VALUE);
		GridPane.setVgrow(textArea, Priority.ALWAYS);
		GridPane.setHgrow(textArea, Priority.ALWAYS);

		GridPane expContent = new GridPane();
		expContent.setMaxWidth(Double.MAX_VALUE);
		//expContent.add(label, 0, 0);
		expContent.add(textArea, 0, 1);

		// Set expandable Exception into the dialog pane.
		alert.getDialogPane().setExpandableContent(expContent);

		alert.showAndWait();
	}
	
	private void showTagSelectionDialog(){
		Dialog<ArrayList<String>> dialog = new Dialog<>();
		dialog.setTitle("Select Tag");
		dialog.getDialogPane().getButtonTypes().addAll(ButtonType.OK, ButtonType.CANCEL);
		TvheadendInfos infos = TvheadendInfos.getInstance();//TODO s'assurer que infos populate
		ObservableList<String> tagsObs = FXCollections.observableArrayList();
		for(Tag tag:infos.getTagList().values()){
			tagsObs.add(tag.getName());
		}
		ListSelectionView<String> lSelection = new ListSelectionView<>();
		lSelection.getSourceItems().addAll(tagsObs);
		//ComboBox<String> cb = new ComboBox<String>(tagsObs);
		dialog.getDialogPane().setContent(lSelection);
		dialog.setResultConverter(dialogButton -> {
		    if (dialogButton == ButtonType.OK) {
		    	ArrayList<String> tags = new ArrayList<>();
		    	for(String str:lSelection.getTargetItems()){
		    		tags.add(str);
		    	}
		        return tags;
		    }
		    return null;
		});
		Optional<ArrayList<String>> ret=dialog.showAndWait();
		/*if(ret!=null){
			for(String tagName:ret){
				System.out.println("tag select "+tagName)
			}
		}*/
		ret.ifPresent(tags->{
			ObservableList<ServiceInfo> selected_ch=tabChannels.getSelectionModel().getSelectedItems();
			 ArrayList<ServiceInfo> servs = new ArrayList<>();
			 for(ServiceInfo serv:selected_ch){
				 servs.add(serv);
			 }
			for(String tagName:tags){
				for(ServiceInfo serv:servs){
					ChannelsManager.addChannel(serv.getServiceUUID(), tagName, ChannelsManager.UUID_MODE);
				}
			}
		
		});
		
	}
	
	public  void removeServiceFromTag(){
		 ObservableList<ServiceInfo> servs_obs = tabChannels.getSelectionModel().getSelectedItems();
		 ArrayList<ServiceInfo> servs = new ArrayList<>();
		 for(ServiceInfo serv:servs_obs){
			 servs.add(serv);
		 }
		String tagName = ((ListView<String>)( (VBox) tagsList.getContent())
				.getChildren().get(0))
				.getSelectionModel().getSelectedItem();
		if (tagName==null){
			System.out.println("no tag selected ");
			return;
		}
		Alert alert = new Alert(AlertType.CONFIRMATION);
		alert.setTitle("Delete Service");
		
		//alert.setContentText("I have a great message for you!");
		//Label label = new Label("The exception stacktrace was:");

		TextArea textArea = new TextArea("Delete "+servs+ " from "+tagName+" ?");
		textArea.setEditable(false);
		textArea.setWrapText(true);

		textArea.setMaxWidth(Double.MAX_VALUE);
		textArea.setMaxHeight(Double.MAX_VALUE);
		GridPane.setVgrow(textArea, Priority.ALWAYS);
		GridPane.setHgrow(textArea, Priority.ALWAYS);

		GridPane expContent = new GridPane();
		expContent.setMaxWidth(Double.MAX_VALUE);
		//expContent.add(label, 0, 0);
		expContent.add(textArea, 0, 1);

		// Set expandable Exception into the dialog pane.
		alert.getDialogPane().setExpandableContent(expContent);
		Optional<ButtonType> result = alert.showAndWait();
		if(result.get() == ButtonType.OK){
			for(ServiceInfo serv:servs){
				
			ChannelsManager.delete(ChannelsManager.DEL_MODE_SERVICE, serv.getServiceUUID(), tagName);
			}
		}
		
	}
	
	private StringProperty getChannelNumber(ServiceInfo serv){
		
		if(serv!=null){
			TvheadendInfos infos = TvheadendInfos.getInstance();
			ChannelConfig chConfig =infos.findChannel(serv.getServiceUUID());
			if(chConfig!=null)
				return new SimpleStringProperty(String.valueOf(chConfig.getNumber()));
			else{
				return new SimpleStringProperty();
			}
		}
	
	
	return null;
	
}
	private StringProperty getChannelName(ServiceInfo serv){
		
			if(serv!=null){
				return new SimpleStringProperty(serv.getSvcname());
			}
		
		
		return null;
		
	}
	
	private StringProperty getChannelFreq(ServiceInfo serv){


		if(serv!=null){
			TvheadendInfos infos = TvheadendInfos.getInstance();
			Muxes mux =infos.getMux(serv.getServiceUUID());
			if(mux!=null){
				return new SimpleStringProperty(String.valueOf(mux.getConfig().getFrequency()/1000));
			}
			return new SimpleStringProperty();
		}


		return null;

	}
	
	private StringProperty getChannelSys(ServiceInfo serv){
		
			if(serv!=null){
				TvheadendInfos infos = TvheadendInfos.getInstance();
				Muxes mux =infos.getMux(serv.getServiceUUID());
				if(mux!=null){
					return new SimpleStringProperty(mux.getConfig().getDelSys());
				}
				return new SimpleStringProperty();
			}
		
		
		return null;
		
	}
	
	private StringProperty getChannelTags(ServiceInfo serv){
		if(serv!=null){
			ArrayList<Tag> tags = new ArrayList<>();
			TvheadendInfos infos = TvheadendInfos.getInstance();
			for(ChannelConfig ch_config:infos.getChannelConfigList().values()){
				for(String servUUID:ch_config.getServices()){
					if(serv.getServiceUUID().equals(servUUID)){
						for(String tagUUID:ch_config.getTags()){
							Tag tag=infos.getTag(tagUUID);
							if(tag!=null){
								tags.add(tag);
							}
						}
			
					}
				}
			}
			StringBuilder strB = new StringBuilder();
			for(Tag tag:tags){
				strB.append(tag.getName()+";");
			}
			return new SimpleStringProperty(strB.toString());
		}
		return new SimpleStringProperty();
	}
	
	private boolean filter = false;
	private void filterByName(String nameFilter){
		if(nameFilter!=null && nameFilter.length()>1){
			saveSorting();
		ObservableList<ServiceInfo> channelsData = FXCollections.observableArrayList();
		if(checkBoxUnencrypted.selectedProperty().get()){
			TvheadendInfos infos = TvheadendInfos.getInstance();
			for(ServiceInfo serv:infos.getListUnencrypted()){
				String name = serv.getSvcname();
				if(name!=null && name.toLowerCase().contains(nameFilter.toLowerCase())){
				channelsData.add(serv);
				}
			}
		}else{
		for(ServiceInfo serv:noFilterChannel){
			String name = serv.getSvcname();
			if(name!=null && name.toLowerCase().contains(nameFilter.toLowerCase())){
			channelsData.add(serv);
			}
		}
		}
	
		updateChannelTab(channelsData);
		filter = true;
		}else if(filter){
			updateChannelTab(noFilterChannel);
			filter = false;
		}
		
		restoreSorting();
		
	/*	//saveSorting();
		// 1. Wrap the ObservableList in a FilteredList (initially display all data).
        FilteredList<ServiceInfo> filteredData = new FilteredList<>(noFilterChannel, p -> true);

        // 2. Set the filter Predicate whenever the filter changes.
        this.nameFilter.textProperty().addListener((observable, oldValue, newValue) -> {
            filteredData.setPredicate(serv -> {
                // If filter text is empty, display all persons.
                if (newValue == null || newValue.isEmpty()) {
                    return true;
                }

                // Compare first name and last name of every person with filter text.
                String lowerCaseFilter = newValue.toLowerCase();
                String name = serv.getSvcname();
                if (name!=null && name.toLowerCase().contains(lowerCaseFilter)) {
                    return true; // Filter matches first name.
                } 
                return false; // Does not match.
            });
        });

        // 3. Wrap the FilteredList in a SortedList. 
        SortedList<ServiceInfo> sortedData = new SortedList<>(filteredData);

        // 4. Bind the SortedList comparator to the TableView comparator.
        sortedData.comparatorProperty().bind(tabChannels.comparatorProperty());

        // 5. Add sorted (and filtered) data to the table.
        tabChannels.setItems(sortedData);
        //restoreSorting();*/
	}
	
	private void filterByProvId(String provID){
		try{
		saveSorting();
		int id = Integer.decode(provID);
		ObservableList<ServiceInfo> channelsData = FXCollections.observableArrayList();
		TvheadendInfos infos = TvheadendInfos.getInstance();
		ArrayList<ServiceInfo> servList = infos.findChannelsWithProverId(id);
	
			channelsData.addAll(servList);
		
		updateChannelTab(channelsData);
		filter = true;
		}catch(NumberFormatException ex){
			updateChannelTab(noFilterChannel);
			filter = false;
		}
		restoreSorting();
	}
	private void deleteTag(String tagName){
		Alert alert = new Alert(AlertType.CONFIRMATION);
		alert.setTitle("Delete Tag");
		alert.setHeaderText("Delete "+tagName+ " ?");
		//alert.setContentText("I have a great message for you!");

		Optional<ButtonType> result = alert.showAndWait();
		if(result.get() == ButtonType.OK){
			ChannelsManager.delete(ChannelsManager.DEL_MODE_TAG, tagName,null);
			
		}
	}
	
	private void updateChannelView(String tagStr){
		TvheadendInfos infos = TvheadendInfos.getInstance();
		ObservableList<ServiceInfo> ch_showed =tabChannels.getItems();
		Tag tag = infos.getTagByName(tagStr);
		if(tag!=null){
			//ObservableList<ServiceInfo> channelsData = FXCollections.observableArrayList();
			List<ChannelConfig> ch_configs = infos.getChannelsTagList(tag.getName());//System.out.println("ch_showed "+ch_showed.size() +" ch_conf "+ch_configs.size());
			for(int i=0;i<ch_showed.size();){
				ChannelConfig c = infos.findChannel(ch_showed.get(i).getServiceUUID());
				if(c!=null){
					if(!ch_configs.contains(c)){
						ServiceInfo rem=ch_showed.remove(i);//System.out.println("remove view "+rem.getSvcname());
						continue;
					}else{
						ch_configs.remove(c);//System.out.println("keep view "+infos.findServiceInfos(c.getServices().get(0)));
					}
				}else{
					ch_showed.remove(i);
					//System.out.println("null for "+ch_showed.get(i).getServiceUUID());
					continue;
				}
				i++;
			}//System.out.println("   ch_showed "+ch_showed.size() +" ch_conf "+ch_configs.size());
			for(ChannelConfig ch_config:ch_configs){
				if(ch_config.getServices().size()>0){
				ch_showed.add(infos.findServiceInfos(ch_config.getServices().get(0)));
				}
				
			}
			updateChannelTab(ch_showed);
			noFilterChannel = ch_showed;
		}
	}
	
	private void saveSorting(){
		if (tabChannels.getSortOrder().size()>0) {
            sortcolumn = (TableColumn) tabChannels.getSortOrder().get(0);
            sortType = sortcolumn.getSortType();
        }
	}
	private void restoreSorting(){
		 if (sortcolumn!=null) {
	            tabChannels.getSortOrder().add(sortcolumn);
	            sortcolumn.setSortType(sortType);
	            sortcolumn.setSortable(true); // This performs a sort
	        }
	}
	
	public void playStream(){
		ServiceInfo serv = tabChannels.getSelectionModel().getSelectedItem();
		if(serv!=null){
			Process process;
			try {
				String path_vlc_bin=null;
				if(OSValidator.isUnix()){
					process = new ProcessBuilder(
							"whereis","vlc").start();

					InputStream is = process.getInputStream();
					InputStreamReader isr = new InputStreamReader(is);
					BufferedReader br = new BufferedReader(isr);
					String line;


					while ((line = br.readLine()) != null) {
						String resp=line.split(":")[1];
						if( resp!=null && resp.length()>0){
							for(String path:resp.split(" ")){
								if(path.startsWith("/usr/bin")){
									path_vlc_bin = path;
									break;

								}
							}
						}
					}
				}else if(OSValidator.isWindows()){
					String path = "C:\\Program Files (x86)\\VideoLAN\\vlc.exe";
					if(new File(path).exists()){
						path_vlc_bin = path;
					}else{
						path = "C:\\Program Files\\VideoLAN\\vlc.exe";
						if(new File(path).exists()){
							path_vlc_bin = path;
						}
					}

				}
				if(path_vlc_bin!=null){
					TvheadendInfos infos = TvheadendInfos.getInstance();
					ChannelConfig ch_config = infos.findChannel(serv.getServiceUUID());
					if(ch_config!=null){
						ChannelConfig serverCh = infos.getServerChannels().get(ch_config.getConfigUUID());
						if(serverCh!=null){
							String uuid = serverCh.getUUID();
							if(uuid!=null){
								new ProcessBuilder(
										path_vlc_bin ,"http://192.168.10.30:9981/play/stream/channel/"+uuid).start();
							}
						}
					}
					return;
				}

			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void editSelection(){
		TvheadendInfos infos = TvheadendInfos.getInstance();
		ObservableList<ServiceInfo> selected_serv = tabChannels.getSelectionModel().getSelectedItems();
		ArrayList<ServiceInfo> servs = new ArrayList<>();
		ArrayList<ChannelConfig> ch_configs = new ArrayList<>();
		for(ServiceInfo serv:selected_serv){
			servs.add(serv);
		}
		for(ServiceInfo serv:servs){
			ChannelConfig ch_conf = infos.findChannel(serv.getServiceUUID());//FIXME bug null exception
			if(ch_conf!=null){
				ch_configs.add(ch_conf);
			}
		}
		
	mainApp.showEditChannel(ch_configs);
	}
}
