package com.sky.tvheadend_manager.ui;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import com.google.gson.Gson;
import com.sky.tvheadend_manager.ChannelConfig;
import com.sky.tvheadend_manager.Settings;
import com.sky.tvheadend_manager.Tag;
import com.sky.tvheadend_manager.TvheadendInfos;

import javafx.application.Application;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.BorderPane;
import javafx.stage.Modality;
import javafx.stage.Stage;

public class MainApp extends Application {

	protected Stage primaryStage;
    private BorderPane rootLayout;
    
    private ChannelListViewController channelController;
    
	@Override
	public void start(Stage primaryStage) {
		this.primaryStage = primaryStage;
        this.primaryStage.setTitle("TvHeadEnd Manager");
        primaryStage.setOnCloseRequest(e -> Platform.exit());
        //this.primaryStage.getIcons().add(new Image(getClass().getResourceAsStream("res/icon.png")));
        initRootLayout();
        
        
        showChannels();
        TvheadendInfos infos = TvheadendInfos.getInstance();
		File setting_file = new File(Settings.SETTING_NAME);
		if(setting_file.exists()){
			FileReader reader;
			try {
				reader = new FileReader(setting_file);
			
			StringBuilder stB = new StringBuilder();
			char[] cbuf = new char[1024];
			while(reader.read(cbuf)!=-1){
				stB.append(cbuf);
			}
			Settings setting = new Gson().fromJson(stB.toString().trim(), Settings.class);
			infos.setTvheadendFolder(setting.getFolder());//TODO fix populate info call 2 times
			infos.setTvheadendServer(setting.getServer());
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}

	public void initRootLayout() {
        try {
            // Load root layout from fxml file.
            FXMLLoader loader = new FXMLLoader();
            loader.setLocation(MainApp.class.getResource("RootLayout.fxml"));
            rootLayout = (BorderPane) loader.load();
            
            ((RootLayoutController)loader.getController()).setMainApp(this);;
            // Show the scene containing the root layout.
            Scene scene = new Scene(rootLayout);
            primaryStage.setScene(scene);
            primaryStage.show();
            scene.addEventHandler(KeyEvent.KEY_PRESSED, (key) -> {
                if(key.getCode()==KeyCode.P) {
                    if(channelController!=null){
                    	channelController.playStream();
                    }
                    return;
                }
                if(key.getCode()==KeyCode.DELETE) {
                    if(channelController!=null){
                    	channelController.removeServiceFromTag();
                    }
                    return;
                }
                
                if(key.getCode()==KeyCode.E) {
                    if(channelController!=null){
                    	channelController.editSelection();;
                    }
                    return;
                }
          });
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
	
	
	public void showChannels() {
        try {
            // Load person overview.
            FXMLLoader loader = new FXMLLoader();
            loader.setLocation(MainApp.class.getResource("ChannelsView.fxml"));
            AnchorPane channelsView = (AnchorPane) loader.load();

            // Set person overview into the center of root layout.
            rootLayout.setCenter(channelsView);
            
            channelController= loader.getController();
            channelController.setMainApp(this);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
	
	public boolean showTagEditDialog(Tag tag) {
	    try {
	        // Load the fxml file and create a new stage for the popup dialog.
	        FXMLLoader loader = new FXMLLoader();
	        loader.setLocation(MainApp.class.getResource("TagEditDialog.fxml"));
	        AnchorPane page = (AnchorPane) loader.load();

	        // Create the dialog Stage.
	        Stage dialogStage = new Stage();
	        if(tag!=null){
	        dialogStage.setTitle("Edit "+tag.toString());
	        }else{
	        	dialogStage.setTitle("Create New tag");
	        }
	        dialogStage.initModality(Modality.WINDOW_MODAL);
	        dialogStage.initOwner(primaryStage);
	        Scene scene = new Scene(page);
	        dialogStage.setScene(scene);

	        // Set the person into the controller.
	        EditTagDialogController controller = loader.getController();
	        controller.setDialogStage(dialogStage);
	        controller.setTag(tag);

	        // Show the dialog and wait until the user closes it
	        dialogStage.showAndWait();

	        //return controller.isOkClicked();
	        return false;
	    } catch (IOException e) {
	        e.printStackTrace();
	        return false;
	    }
	}
	
	
	public void showPreferenceDialog(){
		 try {
		        // Load the fxml file and create a new stage for the popup dialog.
		        FXMLLoader loader = new FXMLLoader();
		        loader.setLocation(MainApp.class.getResource("preference.fxml"));
		        AnchorPane page = (AnchorPane) loader.load();

		        // Create the dialog Stage.
		        Stage dialogStage = new Stage();
		      
		        	dialogStage.setTitle("Preference");
		        
		        dialogStage.initModality(Modality.WINDOW_MODAL);
		        dialogStage.initOwner(primaryStage);
		        Scene scene = new Scene(page);
		        dialogStage.setScene(scene);

		        // Set the person into the controller.
		        PreferenceDialogController controller = loader.getController();
		        controller.setDialogStage(dialogStage);
		        controller.setMainApp(this);
		        controller.setInfos();

		    
		        // Show the dialog and wait until the user closes it
		        dialogStage.showAndWait();

		        //return controller.isOkClicked();
		      
		    } catch (IOException e) {
		        e.printStackTrace();
		        
		    }
	}
	
	public void showEditChannel(ArrayList<ChannelConfig> chConfigs){
		 try {
		        // Load the fxml file and create a new stage for the popup dialog.
		        FXMLLoader loader = new FXMLLoader();
		        loader.setLocation(MainApp.class.getResource("EditChannel.fxml"));
		        AnchorPane page = (AnchorPane) loader.load();

		        // Create the dialog Stage.
		        Stage dialogStage = new Stage();
		      
		        	dialogStage.setTitle("Edit");
		        
		        dialogStage.initModality(Modality.WINDOW_MODAL);
		        dialogStage.initOwner(primaryStage);
		        Scene scene = new Scene(page);
		        dialogStage.setScene(scene);

		        // Set the person into the controller.
		        EditChannelController controller = loader.getController();
		        controller.setMainApp(this);
		        controller.setChannelConfig(chConfigs);

		    
		        // Show the dialog and wait until the user closes it
		        dialogStage.showAndWait();

		        //return controller.isOkClicked();
		      
		    } catch (IOException e) {
		        e.printStackTrace();
		        
		    }
	}
	public static void main(String[] args) {
		launch(args);
		
		//TagListController tagsListView = new TagListController();
		
	}
}
