package com.sky.tvheadend_manager;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;

import org.junit.Test;

import static org.junit.Assert.*;

public class TvheadendMangerTest {
	static final String folder = "files/tvheadend_test";
	@Test public void testCmdParser() {
		String out_console_file="output_console";
		PrintStream out;
		try {
			
			UtilsUnitTest.clearConsoleFile(out_console_file);
			out = new PrintStream(out_console_file);
		
		System.setOut(out);

		CmdParser.parseCmds("list ");
		File f  = new File(out_console_file);
		String ret = UtilsUnitTest.readLineFile(f,0);
		assertEquals("list error","missing parameter",ret);
		
		CmdParser.parseCmds( "unknow ");
		
		 ret = UtilsUnitTest.readLineFile(f,1);
		assertEquals("unknow error","unknow command",ret);
		
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
	}
	
	@Test public void testGetChConfigList() {
		String out_console_file="output_console";
		PrintStream out;
		try {
			
			UtilsUnitTest.clearConsoleFile(out_console_file);
			out = new PrintStream(out_console_file);
		
		System.setOut(out);
		TvheadendInfos infos = TvheadendInfos.getInstance();
		infos.setTvheadendFolder(folder);
		UtilsUnitTest.waitPopulate();
		CmdParser.parseCmds( "list ch_config");
		File f  = new File(out_console_file);
		assertTrue("Config parse error",UtilsUnitTest.findAllOutput("services : {\"sid\":6207,\"lcn\":0,\"lcn_minor\":0,\"lcn2\":0,\"srcid\":0,\"svcname\":\"SFR SPORT 4K\",", f));
		
		
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
	}
	
	@Test public void testGetChTagList() {
		String out_console_file="output_console";
		PrintStream out;
		try {
			;
			UtilsUnitTest.clearConsoleFile(out_console_file);
			out = new PrintStream(out_console_file);
		
		System.setOut(out);
		TvheadendInfos infos = TvheadendInfos.getInstance();
		infos.setTvheadendFolder(folder);
		UtilsUnitTest.waitPopulate();
		CmdParser.parseCmds("list ch_tag BBOX");
		File f  = new File(out_console_file);
		assertTrue("ch tag error",UtilsUnitTest.findAllOutput("services : {\"sid\":7644,\"lcn\":0,\"lcn_minor\":0,\"lcn2\":0,\"srcid\":0,\"svcname\":\"BEIN SPORTS  3 HD\",", f));
		
		
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
	}
	@Test public void testGetTagList() {
		String out_console_file="output_console";
		PrintStream out;
		try {
			UtilsUnitTest.clearConsoleFile(out_console_file);
			out = new PrintStream(out_console_file);
		
		System.setOut(out);
		TvheadendInfos infos = TvheadendInfos.getInstance();
		infos.setTvheadendFolder(folder);
		UtilsUnitTest.waitPopulate();
		CmdParser.parseCmds( "list tags");
		File f  = new File(out_console_file);
		assertTrue("Tag not found",UtilsUnitTest.findAllOutput("SDTV", f));
		
		
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
	}
	
	@Test public void testGetMuxesList() {
		String out_console_file="output_console";
		PrintStream out;
		try {
			UtilsUnitTest.clearConsoleFile(out_console_file);
			out = new PrintStream(out_console_file);
		
		System.setOut(out);
		TvheadendInfos infos = TvheadendInfos.getInstance();
		infos.setTvheadendFolder(folder);
		UtilsUnitTest.waitPopulate();
		CmdParser.parseCmds( "list muxes 4e130385cf891e457db39cb2ce0ff0de");
		File f  = new File(out_console_file);
		/*assertTrue("muxes parse error",findAllOutput("\"svcname\": \"Action HD\",\n\t\t\t\t"+
				"\"provider\": \"ABTV\",\n\t\t\t\t"+
				"\"dvb_servicetype\": 25,", f));*/
		
		assertTrue("muxes parse error",UtilsUnitTest.findAllOutput("Action HD", f));
		
		
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
	}
	
	
	
	
	
	
}
